import pandas as pd
import numpy as np
import matplotlib.pyplot as plt
import scipy.stats as ss
import stan
import seaborn as sb
import arviz as az

pd.options.display.max_rows = 10000

fout = open('Results.txt', 'w')

flnm = input('Data File Name (*.xlsx) = ')
xlsx = pd.ExcelFile(flnm)           
data = pd.read_excel(xlsx)
fout.write(f'Data file = {flnm}\n')

VarLabels = data.columns[1:].values;
print(VarLabels)

X = data.values[:, 1:]
print(X[:10])
N = len(X)
M = len(X[0])
print('N = ', N)
print('M = ', M)
fout.write('\n')
fout.write('VarLables =\n{VarLables}\n')
fout.write(f'N = {N},   M = {M}\n')
fout.write('\nX =\n')
for i, v in enumerate(X):
    fout.write(f'{i+1:>5d}  {v}\n')

Data = {'N': N, 'M': M, 'X': X}

def f_init():
    return dict(mu = [0.0]*M, psy = 1.0,
                F = np.random.normal(size = N))

with open('Reli_2Cat.stan', 'r') as fstan:
    sm = stan.build(fstan.read(), data = Data)
fit = sm.sample(num_chains = 4,init = [f_init()]*4) 

i_data = az.from_pystan(posterior = fit, posterior_model = sm)
smry = az.summary(i_data, var_names = ['mu', 'psy', 'cF', 'mnF', 'sdF'])
print('Summary =\n',smry)
fout.write('\nSummary...\n')
fout.write(f'{smry}\n')

d_fm = fit.to_frame()

print('d_fm...keys...\n', d_fm.keys())

print(fit)

Mus = []
for j in range(M):
    Mus.append(d_fm[f'mu.{j+1}'])
Mus = np.array(Mus).T

for i in range(M):
    sb.kdeplot(Mus.T[i])
plt.title('$\mu$', fontsize = 16)
plt.show()
print()
fout.write('\n')
for j in range(M):
    print('mu[{0:}] = {1:.5}'.format(j+1, np.percentile(Mus.T[j], 50)))
    fout.write('mu[{0:}] = {1:.5}\n'.format(j+1, np.percentile(Mus.T[j], 50)))

Psys = np.array(d_fm['psy'])
sb.kdeplot(Psys)
psy_Q1, psy_Med, psy_Q3 = np.percentile(Psys, [25, 50, 75])
plt.title(f'Mean = {np.mean(Psys):.3f}\nQ1 = {psy_Q1:.3f},  Med.= {psy_Med:.3f}' +
          f',  Q3 = {psy_Q3:.3f}' , fontsize = 16)
plt.xlabel('$\psi$', fontsize = 20)
plt.tight_layout()
plt.show()

print()
fout.write('\n')
print('Psys =\n', Psys)
print('Psy = {0:.5f}\n'.format(np.percentile(Psys, 50)))
fout.write('Psy = {0:.5f}\n'.format(np.percentile(Psys, 50)))

rho = []
for i in range(len(Psys)):
    rho.append(M**2 / (M**2 + M*(Psys[i]**2)))
rho_Q1, rho_Med, rho_Q3 = np.percentile(rho, [25, 50, 75])
rho_Mean = np.array(rho).mean()

print('\nomega(mean) = {0:.5}'.format(rho_Mean))
fout.write('\nomega(mean) = {0:.5}\n'.format(rho_Mean))
print('omega(Med.) = {0:.5}'.format(rho_Med))
fout.write('omega(Med.) = {0:.5}\n'.format(rho_Med))
print('omega(Q1) = {0:.5}     omega(Q3) = {1:.5}'.format(rho_Q1, rho_Q3))
fout.write('omega(Q1) = {0:.5}     omega(Q3) = {1:.5}\n'.format(rho_Q1, rho_Q3))

sb.kdeplot(rho)
plt.title('Mean = {0:.3}\nQ1 = {1:.3}, Med.= {2:.3}, Q3 = {3:.3}'.
          format(rho_Mean, rho_Q1, rho_Med, rho_Q3), fontsize = 14)
plt.xlabel('$\omega$', fontsize = 16)
plt.show()

Xtot = X.sum(axis = 1)
Fs = []
for i in range(N):
    Fs.append(d_fm[f'cF.{i+1}'])
Fs = np.array(Fs).T
Rs = []
R2s = []
for t in range(len(Fs)):
    r = np.corrcoef(Xtot, Fs[t])[0][1]
    Rs.append(r)
    R2s.append(r**2)
    
R_Q1, R_med, R_Q3 = np.percentile(Rs, [25, 50, 75])
R2_Q1, R2_med, R2_Q3 = np.percentile(R2s, [25, 50, 75])
sb.kdeplot(Rs, label = 'R')
sb.kdeplot(R2s, label = '$R^2$')
plt.title('R(Med.) = {0:.3}   $R^2$(Med.) = {1:.3}'.format(R_med, R2_med), fontsize = 16)
plt.legend(fontsize = 16)
plt.show()

FsMeds = np.median(Fs, axis=0)
plt.plot(FsMeds, Xtot, 'o')
plt.xlabel('F', fontsize = 16)
plt.ylabel('X', fontsize = 16)
plt.show()

fout.write('\n')
print('\nR^2(Med.) = {0:.5}'.format(R2_med))
fout.write('\nR^2(Med.) = {0:.5}\n'.format(R2_med))
print('R^2(Q1) = {0:.5}     R^2(Q3) = {1:.5}'.format(R2_Q1, R2_Q3))
fout.write('R^2(Q1) = {0:.5}     R^2(Q3) = {1:.5}\n'.format(R2_Q1, R2_Q3))
print('R(Med.) = {0:.5}'.format(R_med))
fout.write('R(Med.) = {0:.5}\n'.format(R_med))
print('R(Q1) = {0:.5}     R(Q3) = {1:.5}'.format(R_Q1, R_Q3))
fout.write('R(Q1) = {0:.5}     R(Q3) = {1:.5}'.format(R_Q1, R_Q3))

S = np.cov(X, rowvar = False)
VarX = np.var(X.sum(axis = 1))
alpha_coef = (M/(M-1)) * (1 - (np.diag(S).sum() / VarX))
print('\nalpha = {0:.5}'.format(alpha_coef))
fout.write('\n')
fout.write('\nalpha = {0:.5}\n'.format(alpha_coef))

Cor = np.corrcoef(X, rowvar = False)
r_bar = (Cor.sum() - np.diag(Cor).sum()) / (M * (M - 1))
std_alpha = M * r_bar / (1.0 + (M - 1) * r_bar)
print('standardized alpha = {0:.5}'.format(std_alpha))
fout.write('standardized alpha = {0:.5}'.format(std_alpha))

fout.close()
print('Results.txt was saved.')

